(in-package :om)

(in-package :svp)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun make-svp-command1 (srcpath processings begin end
                                 windowsize fftsize windowstep-oversamp
                                 window-type shape-invariant preserve-transient
                                 normalize audio-res extend-across
                                 outpath)
  
  (let ((supervp-path (om::svp-path)))
    
    (if (and supervp-path (probe-file supervp-path))
        (let ((srcpath (pathname srcpath))
              (beginstr (if begin (format nil "-B~f " begin) "")) 
              (endstr (if end (format nil "-E~f " end) "")) 
              (command ""))
          (when (probe-file srcpath)
             
            (setf command (om::string+ command (format nil "~A -t -Z " (namestring supervp-path))))
             
            (when normalize (setf command (om::string+ command "-norm ")))
            
            ;;; in AS when "Extend across sound limits" is selected (default in AS), the -U disappears...
            (when (null extend-across) (setf command (om::string+ command "-U ")))
            
            (setf command (om::string+ command (format nil "-S~A -Afft " (namestring srcpath))))

            (setf command (om::string+ command beginstr endstr))
             
            (setf command (om::string+ command (format nil "-M~D -N~D -W~D " windowsize fftsize  window-type)
                                       (if windowstep-oversamp (format nil "-oversamp ~D " windowstep-oversamp ""))))

            (when shape-invariant (setf command (om::string+ command "-shape 2 -Vuf -4 ")))
        
            ;;; transients
            (if preserve-transient
                (let ((tr-settings (when preserve-transient
                                     (if (listp preserve-transient) preserve-transient
                                       '(1.4 (0.0 22050.0) 1.5 nil)))))

                  (setf command (om::string+ command (format nil "-P1 -td_thresh ~f -td_band ~f,~f -td_ampfac ~f "
                                                             (nth 0 tr-settings)
                                                             (car (nth 1 tr-settings))
                                                             (cadr (nth 1 tr-settings))
                                                             (nth 2 tr-settings)
                                                             )))
                  (when (nth 4 tr-settings)
                    (setf command (om::string+ command (format nil "-td_relaxto ~f -td_relax ~f ")
                                               (car (nth 3 tr-settings))
                                               (cadr (nth 3 tr-settings))
                                               )))
                  )
              (unless (find "-Ffshift" (om::list! processings) :test #'(lambda (s1 s2) 
                                                                         (and (>= (length s2) 8)  
                                                                              (string-equal s1 s2 :end1 8 :end2 8))))
                ;;; option -P0 (phase synchronization) incompatible avec freq shifting
                (setf command (om::string+ command (format nil "-P0 "))))
              )
        
            ;;; processings
            (loop for p in (om::list! processings) do (setf command (om::string+ command p " ")))
        
            ;;; out
            (setf command (om::string+ command (format nil "-Osa~D ~A" audio-res (namestring outpath))))
        
            command))
  
      (om::om-beep-msg "SuperVP not found! Set path to SuperVP in the OM preferences.")
      )))



(defun write-svp-cndline (path cmdline)
  (let ((pathname (or path (om-choose-new-file-dialog))))
    (WITH-OPEN-FILE (out pathname :direction :output  :if-does-not-exist :create :if-exists :supersede)
     (format out "~A"  cmdline)
      )
    pathname))


(defmethod! om::svp-proc ((src pathname) processings begin end windowsize fftsize windowstep-oversamp 
                                    window-type shape-invariant preserve-transient
                                    normalize outfile &optional (extend-across nil))
  :icon 950 
  :menuins '((6 (("1/4" 4) ("1/8" 8) ("1/16" 16) ("1/32" 32)))
             (7 (("Blackman" "blackman") ("Hanning" "hanning")("Hamming" "hamming")))
             (8 (("Shape Invariant On" t) ("Shape Invariant Off" nil)))
             (10 (("Normalize On" t) ("Normalize Off" nil)))
             (12 (("Yes" t) ("No" nil))))
  :indoc '("input sound" 
           "treatment (or list of treatments)" 
           "begin time (s)" "end time (s)" 
           "size of analysis window" "FFT size" "overlapping factor" 
           "type of analysis window (select from menu)"
           "" "" "" "output file name" "exetdn across sound limits")
  :initvals '(nil "" nil nil 4096 4096 8 "hanning" nil t nil "out.aiff" nil)
(let* (
       (outpath (om::handle-new-file-exists (if (pathnamep outfile) outfile (om::outfile outfile))))
       (cmd (make-svp-command1 src processings begin end windowsize fftsize windowstep-oversamp 
                              window-type shape-invariant preserve-transient normalize 16
                              extend-across
                              outpath)))
    (when cmd
      (print "==========================")
      (print "SUPERVP PROCESSING")
      (print "==========================")
      (print (format nil "toto: ~A" cmd))
      (print (format nil "Input file: ~s" (namestring src)))
      (print (format nil "Output file: ~s" (namestring outpath)))
      (write-svp-cndline (om::outfile (format nil "~A_cmdline.txt" outfile)) cmd)
      (om::om-cmd-line cmd)
      (om::maybe-clean-tmp-files)
      (probe-file outpath))))


(defmethod! om::svp-proc ((src om::sound) processings begin end windowsize fftsize windowstep-oversamp 
                                     window-type shape-invariant preserve-transient
                                     normalize outfile &optional (extend-across nil))
     (if (om::om-sound-file-name src)           
         (om::svp-proc (om::om-sound-file-name src) 
                             processings begin end windowsize fftsize windowstep-oversamp 
                             window-type shape-invariant preserve-transient normalize
                             outfile 
                             extend-across)
         (om::om-beep-msg "SUPERVP-PROCESSING: input sound file must be saved on disk!")))